CHEMICAL_ELEMENTS = [
    "Hydrogen", "Helium", "Lithium", "Beryllium", "Boron", "Carbon", "Nitrogen", "Oxygen", "Fluorine", "Neon",
    "Sodium", "Magnesium", "Aluminum", "Silicon", "Phosphorus", "Sulfur", "Chlorine", "Argon",
    "Potassium", "Calcium", "Scandium", "Titanium", "Vanadium", "Chromium", "Manganese", "Iron", "Cobalt", "Nickel", "Copper", "Zinc",
    "Gallium", "Germanium", "Arsenic", "Selenium", "Bromine", "Krypton",
    "Rubidium", "Strontium", "Yttrium", "Zirconium", "Niobium", "Molybdenum", "Technetium", "Ruthenium", "Rhodium", "Palladium", "Silver", "Cadmium",
    "Indium", "Tin", "Antimony", "Tellurium", "Iodine", "Xenon",
    "Cesium", "Barium", "Lanthanum", "Cerium", "Praseodymium", "Neodymium", "Promethium", "Samarium", "Europium", "Gadolinium", "Terbium", "Dysprosium", "Holmium", "Erbium", "Thulium", "Ytterbium", "Lutetium",
    "Hafnium", "Tantalum", "Tungsten", "Rhenium", "Osmium", "Iridium", "Platinum", "Gold", "Mercury",
    "Thallium", "Lead", "Bismuth", "Polonium", "Astatine", "Radon",
    "Francium", "Radium", "Actinium", "Thorium", "Protactinium", "Uranium", "Neptunium", "Plutonium", "Americium", "Curium", "Berkelium", "Californium", "Einsteinium", "Fermium", "Mendelevium", "Nobelium", "Lawrencium",
    "Rutherfordium", "Dubnium", "Seaborgium", "Bohrium", "Hassium", "Meitnerium", "Darmstadtium", "Roentgenium", "Copernicium", "Nihonium", "Flerovium", "Moscovium", "Livermorium", "Tennessine", "Oganesson"
]

CHEMICAL_ELEMENT_SYMBOLS = [
    "H", "He", "Li", "Be", "B", "C", "N", "O", "F", "Ne",
    "Na", "Mg", "Al", "Si", "P", "S", "Cl", "Ar",
    "K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn",
    "Ga", "Ge", "As", "Se", "Br", "Kr",
    "Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd",
    "In", "Sn", "Sb", "Te", "I", "Xe",
    "Cs", "Ba", "La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb", "Dy", "Ho", "Er", "Tm", "Yb", "Lu",
    "Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg",
    "Tl", "Pb", "Bi", "Po", "At", "Rn",
    "Fr", "Ra", "Ac", "Th", "Pa", "U", "Np", "Pu", "Am", "Cm", "Bk", "Cf", "Es", "Fm", "Md", "No", "Lr",
    "Rf", "Db", "Sg", "Bh", "Hs", "Mt", "Ds", "Rg", "Cn", "Nh", "Fl", "Mc", "Lv", "Ts", "Og"
]

# --- Physics & Chemistry ---

# Speed of light in vacuum (m/s)
SPEED_OF_LIGHT = 299792458.0

# Planck constant (J·s)
PLANCK_CONSTANT = 6.62607015e-34

# Avogadro's number (mol⁻¹)
AVOGADRO_NUMBER = 6.02214076e23

# Universal gas constant (J/(mol·K))
UNIVERSAL_GAS_CONSTANT = 8.314462618

# Boltzmann constant (J/K)
BOLTZMANN_CONSTANT = 1.380649e-23

# Gravitational constant (m³·kg⁻¹·s⁻²)
GRAVITATIONAL_CONSTANT = 6.67430e-11

# Standard acceleration of gravity on Earth (m/s²)
STANDARD_GRAVITY = 9.80665

# Elementary charge (C)
ELEMENTARY_CHARGE = 1.602176634e-19

# Electron mass (kg)
ELECTRON_MASS_KG = 9.1093837015e-31

# Proton mass (kg)
PROTON_MASS_KG = 1.67262192369e-27

# Absolute zero (in Celsius)
ABSOLUTE_ZERO_CELSIUS = -273.15

# --- Astronomy ---

# Astronomical Unit, the mean Earth-Sun distance (meters)
ASTRONOMICAL_UNIT_KM = 149597870.7

# Light-year (meters)
LIGHT_YEAR_KM = 9460730472580.8

# Earth's equatorial radius (meters)
EARTH_RADIUS_KM = 6378.137

# Mass of the Earth (kg)
EARTH_MASS_KG = 5.9722e24

# Mass of the Sun (kg)
SUN_MASS_KG = 1.98847e30
